(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory(require('@fortawesome/fontawesome')) :
	typeof define === 'function' && define.amd ? define(['@fortawesome/fontawesome'], factory) :
	(global['vue-fontawesome'] = factory(global.FontAwesome));
}(this, (function (fontawesome) { 'use strict';

fontawesome = fontawesome && fontawesome.hasOwnProperty('default') ? fontawesome['default'] : fontawesome;

'use strict';

function preserveCamelCase(str) {
	var isLastCharLower = false;
	var isLastCharUpper = false;
	var isLastLastCharUpper = false;

	for (var i = 0; i < str.length; i++) {
		var c = str[i];

		if (isLastCharLower && /[a-zA-Z]/.test(c) && c.toUpperCase() === c) {
			str = str.substr(0, i) + '-' + str.substr(i);
			isLastCharLower = false;
			isLastLastCharUpper = isLastCharUpper;
			isLastCharUpper = true;
			i++;
		} else if (isLastCharUpper && isLastLastCharUpper && /[a-zA-Z]/.test(c) && c.toLowerCase() === c) {
			str = str.substr(0, i - 1) + '-' + str.substr(i - 1);
			isLastLastCharUpper = isLastCharUpper;
			isLastCharUpper = false;
			isLastCharLower = true;
		} else {
			isLastCharLower = c.toLowerCase() === c;
			isLastLastCharUpper = isLastCharUpper;
			isLastCharUpper = c.toUpperCase() === c;
		}
	}

	return str;
}

var camelcase = function (str) {
	if (arguments.length > 1) {
		str = Array.from(arguments)
			.map(function (x) { return x.trim(); })
			.filter(function (x) { return x.length; })
			.join('-');
	} else {
		str = str.trim();
	}

	if (str.length === 0) {
		return '';
	}

	if (str.length === 1) {
		return str.toLowerCase();
	}

	if (/^[a-z0-9]+$/.test(str)) {
		return str;
	}

	var hasUpperCase = str !== str.toLowerCase();

	if (hasUpperCase) {
		str = preserveCamelCase(str);
	}

	return str
		.replace(/^[_.\- ]+/, '')
		.toLowerCase()
		.replace(/[_.\- ]+(\w|$)/g, function (m, p1) { return p1.toUpperCase(); });
};

function styleToObject (style) {
  return style.split(';')
    .map(function (s) { return s.trim(); } )
    .filter(function (s) { return s; })
    .reduce(function (acc, pair) {
      var i = pair.indexOf(':');
      var prop = camelcase(pair.slice(0, i));
      var value = pair.slice(i + 1).trim();
      
      acc[prop] = value;

      return acc
    }, {})
}

function classToObject (cls) {
  return cls.split(/\s+/)
    .reduce(function (acc, c) {
      acc[c] = true;
      
      return acc
    }, {})
}

function convert (h, element, props) {
  if ( props === void 0 ) props = {};

  var children = (element.children || []).map(convert.bind(null, h));

  var mixins = Object.keys(element.attributes).reduce(function (acc, key) {
    var val = element.attributes[key];

    switch (key) {
      case 'class':
        acc['class'] = classToObject(val);
        break
      case 'style':
        acc['style'] = styleToObject(val);
        break
      default:
        acc.attrs[key] = val;
    }

    return acc
  }, { 'class': {}, style: {}, attrs: {} });

  return h(
    element.tag,
    {
      class: mixins.class,
      style: mixins.style,
      attrs: mixins.attrs,
      props: props
    },
    children
  )
}

var FontAwesomeIcon$1 = {
  name: 'FontAwesomeIcon',

  props: {
    border: {
      type: Boolean,
      default: false
    },
    fixedWidth: {
      type: Boolean,
      default: false
    },
    flip: {
      type: String,
      default: null,
      validator: function (value) { return ['horizontal', 'vertical', 'both'].indexOf(value) > -1; }
    },
    iconDefinition: {
      type: Object,
      default: null
    },
    listItem: {
      type: Boolean,
      default: false
    },
    pack: {
      type: String,
      default: 'fa'
    },
    pull: {
      type: String,
      default: null,
      validator: function (value) { return ['right', 'left'].indexOf(value) > -1; }
    },
    pulse: {
      type: Boolean,
      default: false
    },
    name: {
      type: String,
      default: ''
    },
    rotation: {
      type: Number,
      default: null,
      validator: function (value) { return [90, 180, 270].indexOf(value) > -1; }
    },
    size: {
      type: String,
      default: null,
      validator: function (value) { return ['lg', 'xs', 'sm', '1x', '2x', '3x', '4x', '5x', '6x', '7x', '8x', '9x', '10x'].indexOf(value) > -1; }
    },
    spin: {
      type: Boolean,
      default: false
    },
    transform: {
      type: [String, Object],
      default: null
    }
  },

  data: function data () {
    return {
      packNames: {
        brands: 'fab',
        light: 'fal',
        regular: 'far',
        solid: 'fas'
      }
    }
  },

  computed: {
    prefix: function prefix () {
      return this.packNames[this.pack] || this.pack
    },

    iconConfig: function iconConfig () {
      return { prefix: this.prefix, iconName: this.name }
    },

    classList: function classList () {
      var classes = {
        'fa-spin': this.spin,
        'fa-pulse': this.pulse,
        'fa-fw': this.fixedWidth,
        'fa-border': this.border,
        'fa-li': this.listItem,
        'fa-flip-horizontal': this.flip === 'horizontal' || this.flip === 'both',
        'fa-flip-vertical': this.flip === 'vertical' || this.flip === 'both'
      };
      classes[("fa-" + (this.size))] = this.size !== null;
      classes[("fa-rotate-" + (this.rotation))] = this.rotation !== null;
      classes[("fa-pull-" + (this.pull))] = this.pull !== null;

      return Object.keys(classes)
        .map(function (key) { return classes[key] ? key : null; })
        .filter(function (key) { return key; })
    },

    transformDirectives: function transformDirectives () {
      return (typeof this.transform === 'string') ? fontawesome.parse.transform(this.transform) : this.transform
    },

    icon: function icon () {
      var params = Object.assign({},
        this.classList.length > 0 && {classes: this.classList},
        this.transformDirectives && {transform: this.transformDirectives}
      );

      return fontawesome.icon(this.iconDefinition || this.iconConfig, params)
    }
  },

  render: function render (createElement) {
    var ref = this.icon;
    var abstract = ref.abstract;
    var convertCurry = convert.bind(null, createElement);

    return convertCurry(abstract[0])
  }
};

return FontAwesomeIcon$1;

})));
